package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

/**
 * Convenience class for accessing DOM elements and attributes. 
 * Methods return the data or null/empty result objects if not 
 * found. Not intended for public use.
 * 
 * @version %I%
 */
public class DOMDataRetriever implements Serializable {
		private final String EMPTY_STRING = "";
		
		DOMDataRetriever() {
			super();
		}	
		
		// return the value of theAttribute from each of the tagName elements 		
		Vector getAttributeFromElements(Document document, String tagName, String theAttribute) {
			NodeList list = document.getElementsByTagName(tagName);
			int length = list.getLength();
						
			if (!(length > 0)) {
				return null;
			}	
			
			Element element;
			Vector result = new Vector();
						
			for (int i = 0; i < length; i++) {
				element = (Element) list.item(i);
				String attribute = (element.getAttribute(theAttribute)).trim();
				if (!attribute.equals(EMPTY_STRING)) {				
					result.add(attribute);				
				}
			}								
			if (result.isEmpty()) {
				return null;
			}	
			
			result.trimToSize();	
			return result;
		}	
		
		// return the value of theAttribute from a singleton tagName element 
		String getAttributeFromSingleElement(Document document, String tagName, String theAttribute) {
			String result = null;
			NodeList list = document.getElementsByTagName(tagName);
		
			if (list.getLength() == 1) {
				String attribute = (((Element)(list.item(0))).getAttribute(theAttribute)).trim();			
				if (!attribute.equals(EMPTY_STRING)) {					
					result = attribute;
				}	
			} 		
			return result;		
		}	
		
		// return the number of tagName elements
		int getNumberOfElements(Document document, String tagName) {
			NodeList list = document.getElementsByTagName(tagName);
			return (list.getLength());
		}	
			
		// return the attribute names and values from a singleton tagName element 
		Hashtable getAttributesFromSingleElement(Document document, String tagName) {
			Hashtable result = null;
			NodeList list = document.getElementsByTagName(tagName);
			
			if (list.getLength() != 1) {
				return result;
			}	
			
			NamedNodeMap attrs = list.item(0).getAttributes();
			int number = attrs.getLength();
			if (number > 0) {
				result = new Hashtable(number);			
				for (int i = 0; i < number; i++) {
					String name = (((Attr) attrs.item(i)).getName()).trim();
					String value = (((Attr) attrs.item(i)).getValue()).trim();	
					// NullPointerException not possible														
					if (!name.equals(EMPTY_STRING) && !value.equals(EMPTY_STRING)) {
						result.put(name, value);						
					}
				}
				if (result.isEmpty()) {
					return null;
				}		
			}			
			return result;	
		}	
		
		// return the value of theAttribute from each of childElementName elements which are children of singleton tagName element  
		Vector getAttributeValuesFromSingleElementChildren(Document document, String tagName, String childElementName, String theAttribute) {
			Vector result = null;
			NodeList list = document.getElementsByTagName(tagName);
			
			
			if (list.getLength() > 0) {			
				Element element = (Element)(list.item(0));
				if (element.getTagName().equals(tagName)) {
					list = element.getElementsByTagName(childElementName);
					int length = list.getLength();			
					if (length > 0) {		
						result = new Vector();
						for (int i = 0; i < length; i++) {
							Element e = (Element)list.item(i);							
							String attribute = (e.getAttribute(theAttribute)).trim();							
							if (!attribute.equals(EMPTY_STRING)) {
								result.add(attribute);									
							}																					
						}
						if (result.isEmpty()) {
							return null;
						}				
						result.trimToSize();	
					}
				}
				
			}				
			return result;					
		}	
		
		// return the value of theAttribute from each of childElementName elements which are children of the first returned tagName element 
		Hashtable getAttributeValuesFromChildrenOfFirstElement(Document document, String tagName, String childElementName, String childAttributeKey, String childAttributeValue) {
			Hashtable result = null;
						
			// get the elements with the elementName we want
			NodeList list = document.getElementsByTagName(tagName);
			if (list.getLength() > 0) {			
				// take the first one
				Element theElement = (Element)(list.item(0));
				// get all the child elements we are looking for			
				if (theElement != null) {
					list = theElement.getElementsByTagName(childElementName);	
					int length = list.getLength();
					if (length > 0) {						
						result = new Hashtable(length);									
						for (int i = 0; i < length; i++) {
							Element e = (Element)list.item(i);
							// put the attributes we are looking for in a hash table							
							String key = (e.getAttribute(childAttributeKey)).trim();
							String value = (e.getAttribute(childAttributeValue)).trim();
							// NullPointerException not possible
							if (!key.equals(EMPTY_STRING) && !value.equals(EMPTY_STRING)) {
								result.put(key, value);									
							}
						}
						if (result.isEmpty()) {
							return null;
						}					
					}							
				}	
			} 			
			return result;		
		}	
		
		// return the value of theAttribute from each of childElementName elements which are children of the tagName element with the docOrder ordinal position
		Hashtable getAttributeValuesFromChildrenOfElementWithDocOrderOrdinal(Document document, String tagName, String childElementName, String childAttributeKey, String childAttributeValue, String ordinal) {
			Hashtable result = null;
			int position = Integer.parseInt(ordinal);			
			// get the elements with the elementName we want
			NodeList list = document.getElementsByTagName(tagName);
			int parentLength = list.getLength();
			if (parentLength > position) {			
				// get the one with the doc order ordinal
				Element theElement = (Element)(list.item(position));
				// get all the child elements we are looking for			
				if (theElement != null) {
					list = theElement.getElementsByTagName(childElementName);	
					int length = list.getLength();
					if (length > 0) {						
						result = new Hashtable(length);									
						for (int i = 0; i < length; i++) {
							Element e = (Element)list.item(i);
							// put the attributes we are looking for in a hash table							
							String key = (e.getAttribute(childAttributeKey)).trim();
							String value = (e.getAttribute(childAttributeValue)).trim();
							// NullPointerException not possible
							if (!key.equals(EMPTY_STRING) && !value.equals(EMPTY_STRING)) {
								result.put(key, value);									
							}
						}
						if (result.isEmpty()) {
							return null;
						}					
					}							
				}	
			} 			
			return result;		
		}	
		
		// 
		Hashtable getChildAttributesFromElementWithAttribute(Document document, String tagName, String elementAttribute, String elementAttributeValue, String childElementName, String childAttributeKey, String childAttributeValue) {
			Hashtable result = null;
			Element theElement = null;
			
			// get the elements with the elementName we want
			NodeList list = document.getElementsByTagName(tagName);
			if (list.getLength() > 0) {				
				// get the element with the attribute value we are looking for
				for (int i = 0, length = list.getLength(); i< length; i++) {				
					String attr = ((Element)(list.item(i))).getAttribute(elementAttribute);
					if (attr.equals(elementAttributeValue)) {
						theElement = (Element)(list.item(i));
					}					
					else if (attr.equals(EMPTY_STRING) && length == 1) {
						theElement = (Element)(list.item(i));	
						break;
					}
				}				
				// get all the child elements we are looking for
				if (theElement != null) {
					list = theElement.getElementsByTagName(childElementName);	
					int length = list.getLength();
					if (length > 0) {
						result = new Hashtable(length);												
						for (int i = 0; i < length; i++) {
							Element e = (Element)list.item(i);							
							// NullPointerException not possible
							String key = (e.getAttribute(childAttributeKey)).trim();
							String value = (e.getAttribute(childAttributeValue)).trim();
							if (!key.equals(EMPTY_STRING) && !value.equals(EMPTY_STRING)) {
								result.put(key, value);
							}								
						}
						if (result.isEmpty()) {
							return null;
						}								
					}				
				}				
			}				
			return result;
		}	

		// 
		String getAttributeValueFromFirstChildOfFirstElement(Document document, String tagName, String childElementName, String childAttributeName) {
			String result = null;			
			// get the elements with the elementName we want
			NodeList list = document.getElementsByTagName(tagName);
			if (list.getLength() > 0) {			
				// take the first one
				Element theElement = (Element)(list.item(0));
				// get all the child elements we are looking for			
				if (theElement != null) {
					list = theElement.getElementsByTagName(childElementName);	
					int length = list.getLength();
					if (length > 0) {					
						// take the first one
						Element childElement = (Element)(list.item(0));
						if (childElement != null) {
							String value = (childElement.getAttribute(childAttributeName)).trim();
							if (!value.equals(EMPTY_STRING)) {
								result = value;
							}	
						}
					}
				}			
			}					
			return result;				
		}
}