package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

/**
 * Signals that the XML data cannot be processed by the consuming widget.
 * 
 * This exception will be raised when the widget request to retrieve
 * the required data from the XML <code>Document</code> object fails.
 * An error code is provided for, along with the tag name of the problem
 * XML element and the widget type consuming the data. 
 * 
 * @version %I%
 * 
 * @see IDataInputProcessingExceptionCodes
 * @see IGraphicDocumentGenerator 
 */
public class DataInputProcessingException extends Exception {
	private int errorCode = 0;
	private String sourceElement = null;
	private String widgetType = null;
	
	/**
	 * Default constructor.
	 */
	public DataInputProcessingException() { }
	
	/**
	 * Constructs a <code>DataInputProcessingException</code> with the
	 * specified detail message.
	 * 
	 * @param msg a message providing information about the exception.
	 * @param element the tag name of the data <code>Element</code>
	 */
	public DataInputProcessingException(String msg, String element) { 
		super(msg);
		this.sourceElement = element;
		
	}
	
	/**
	 * Constructs a <code>DataInputProcessingException</code> with the
	 * specified error code and the data element tag name.
	 * 
	 * @param error one of the documented error codes.
	 * @param element the tag name of the data <code>Element</code>
	 */
	public DataInputProcessingException(int error, String element) {
		this.errorCode = error;
		this.sourceElement = element;		
	}
	/**
	 * Constructs a <code>DataInputProcessingException</code> with the
	 * specified detail message, the error code and the data element tag
	 * name.
	 * 
	 * @param msg a message providing information about the exception.
	 * @param error one of the documented error codes
	 * @param element the tag name of the data <code>Element</code>
	 */
	public DataInputProcessingException(String msg, int error, String element) {
		super(msg);
		this.errorCode = error;
		this.sourceElement = element;		
	}
	
	/**
	 * Constructs a <code>DataInputProcessingException</code> with the
	 * specified detail message, the error code, the data element tag 
	 * name and the widget type.
	 * 
	 * @param msg a message providing information about the exception.
	 * @param error one of the documented error codes
	 * @param element the tag name of the data <code>Element</code>
	 * @param type the graphic type produced by the object throwing the exception  
	 */
	public DataInputProcessingException(String msg, int error, String element, String type) {
		super(msg);
		this.errorCode = error;
		this.sourceElement = element;
		this.widgetType = type;
	}
	
	/**
	 * Returns the error code which was passed as constructor parameter. 
	 */		
	public int getErrorCode() {
		return errorCode;
	}
		
	/**
	 * Returns the element which was passed as constructor parameter. 
	 */
	public String getElement() {
		return sourceElement;
	}
		
	/**
	 * Returns the graphic type which was passed as constructor parameter. 
	 */
	public String getWidgetType() {
		return widgetType;
	}	
	
	/**
	 * Sets the graphic type produced by the object throwing the exception.
	 * 
	 * @param type the type of graphic 
	 */
	public void setWidgetType(String type) {
		this.widgetType = type;
	}			
}	

