package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * <code>ECMAScriptGenerator</code> generates the (ECMA 1.0-compliant) JavaScript functions
 * needed to support browser-based user interaction with <code>GraphicDocumentGenerators</code> 
 * that generate Scalable Vector Graphics (SVG).
 * 
 * @version %I%
 * 
 */
public class ECMAScriptGenerator {
	private StringBuffer script;
	
	/** 
	 * Generates browser information variables for use in browser specific code.
	 */
	public String generateClientSnifferVariables(){
		script = new StringBuffer("var browserVersion = parseInt(parent.navigator.appVersion); ");		
		script.append("var isNN = parent.navigator.appName.indexOf(\"Netscape\")!= -1; ");
		script.append("var isIE = parent.navigator.appName.indexOf(\"Microsoft\")!= -1; ");		
				
		return script.toString();
	}	
	
	/** 
	 * Generates global variables for use in HTML functions.
	 */
	public String generateGlobalVariables(){		
		script = new StringBuffer("var currentValues = null; ");
		script.append("var palettes = null; ");
		script.append("var preferencePage = null;");	
		return script.toString();
	}
	
	/** 
	 * Generates an event handler which performs initialization functions.
	 * Typically assigned as the "onload" event handler. 
	 * 
	 * @param palettes the available data set color palettes 
	 * @param type		the graphic type
	 */
	public String generateInitFunction(Hashtable palettes, String currentPalette, String currentFont, String type){
		script = new StringBuffer("function init(){");
		
		if (palettes == null || palettes.isEmpty()) {
			script.append("} ");
			return script.toString();			
		}			
		
		// create the colour palettes array		
		script.append("palettes = new Array();");
		
		Enumeration keys = palettes.keys();
		int i = 0;		
		while (keys.hasMoreElements()) {
			String name = (String)keys.nextElement();
			String [] codes = (String [])palettes.get(name);	
			
			script.append("palettes[");
			script.append(i);
			script.append("] = [");			
			// add the colour codes
			for (int j = 0; j < codes.length; j++) {
				script.append("\"");
				script.append(codes[j]);
				script.append("\", ");				
			}		
			
			// add the palette name	
			script.append("\"");	
			script.append(name);
			script.append("\"");	
			script.append("];");
			i++;
		}
			
		// get any saved user preferences 
		script.append("currentValues = getPreferences();");
	    script.append("if (currentValues != null) {");		
		script.append("updateStyles(currentValues, false);}");	
		
		// OR initialize the current values with the default settings		
		String defaultFont = IGraphicDocumentStyle.BASIC_FONTSTYLE;
		String newDefaultFont = defaultFont.substring(0, defaultFont.indexOf("font-family") + "font-family".length() + 1)
								+ currentFont 
								+ defaultFont.substring(defaultFont.indexOf("; fill"), defaultFont.length());
				
		script.append("else {currentValues = [\"");
		script.append(currentPalette);
		script.append("\",\"");
		script.append(IGraphicDocumentStyle.BASIC_FONTSIZE);		
		script.append("\",\"");
		script.append(newDefaultFont);
		script.append("\",\"");
		script.append(IGraphicDocumentStyle.BASIC_SHOW_SHAPES);
		script.append("\",\"");
		// default for show values in pie chart is ON, all others OFF
		if (type == IGraphicTypeConstants.PIE_CHART) {
			script.append("1");
		}
		else {	
			script.append(IGraphicDocumentStyle.BASIC_SHOW_VALUES);
		}
		script.append("\",\"");
		script.append(IGraphicDocumentStyle.BASIC_SHOW_PERCENTS);
		script.append("\"];updateStyles(currentValues, true);}");				
		script.append("} ");		
					
		return script.toString();
	}	
	
	/** 
	 * Generates an event handler which opens a new browser window on the content "url" + "src" 
	 * with the specified "name", "width" and "height". 
	 * 
	 * @param url		the URL to the folder which contains "src"
	 * @param src		the content file
	 * @param name		the name of the new window
	 * @param width	the width of the new window
	 * @param height	the height of the new window
	 */	
	public String generateOpenPreferencePageFunction(String url, String src, String name, String width, String height){
		script = new StringBuffer("function openPreferencePage(){");	
		// needed for basic types preference pages if using NS 4.7
		if (src.indexOf(IGraphicDocumentStyle.PREFS) == 0) {	
			script.append("if (isNN) {useNetscapePreferencePage();}");
			script.append("else {");	
		}
		script.append("if (preferencePage) {preferencePage = null;}");
		script.append("preferencePage=open('");	
		script.append(url + src);		
		script.append("','");		
		script.append(name);
		script.append("', 'width=");		
		script.append(width);
		script.append(",height=");		
		script.append(height);
		script.append("'); ");
		script.append("preferencePage.opener=window;} ");		
		script.append("} ");
		// might want to move the preference page so it isn't blocking the widget, if possible
              
		return script.toString();
	}		
		
	/** 
	 * Generates a function which launches the NS 4.7 preference page.
	 * Extension classes should not use this for custom preference pages. 	
	 * 
	 * @param url		the URL to the folder which contains "src"
	 * @param src		the content file
	 * @param name		the name of the new window
	 */
	public String generateUseNetscapePreferencePage(String url, String src, String name, String pageWidth, String pageHeight){					
		String height = null;
		String ns_src = IGraphicDocumentStyle.NS_PREFS;
		String width = IGraphicDocumentStyle.NS_PREFS_WIDTH;
			
		if (name.equals(IGraphicTypeConstants.LINE_CHART)) {
			height = IGraphicDocumentStyle.NS_LINE_PREFS_HEIGHT;	
		} else if (name.equals(IGraphicTypeConstants.BAR_CHART)) {
			height = IGraphicDocumentStyle.NS_BAR_PREFS_HEIGHT;
		} else if (name.equals(IGraphicTypeConstants.STACKBAR_CHART)) {
			height = IGraphicDocumentStyle.NS_STACKBAR_PREFS_HEIGHT;
		} else if (name.equals(IGraphicTypeConstants.PIE_CHART)) {
			height = IGraphicDocumentStyle.NS_PIE_PREFS_HEIGHT;
		} else if (name.equals(IGraphicTypeConstants.METER)) {
			height = IGraphicDocumentStyle.NS_METER_PREFS_HEIGHT;
		} else {
			width = pageWidth;
			height = pageHeight;			
		}
		
		script = new StringBuffer("function useNetscapePreferencePage(){");					
		script.append("if (preferencePage) {preferencePage = null;}");
		script.append("preferencePage=open('");	
		script.append(url + ns_src);		
		script.append("','");
		script.append(name);
		script.append("','");
		script.append("width=");
		script.append(width);
		script.append(",");
		script.append("height=");
		script.append(height);
		script.append(",top=0,left=0,screenx=0,screeny=0,toolbar=no,scrollbar=no'); ");
		script.append("preferencePage.prefType='");
		script.append(name);
		script.append("'; preferencePage.opener=window;");						
		script.append("}");	
		
		return script.toString();
	}			
			
		
	/** 
	 * Generates a function which returns the set of available palettes. 	
	 */
	public String generateGetPalettesFunction(){
		script = new StringBuffer("function getPalettes(){");		
		script.append("return palettes;");
		script.append("} ");
		
		return script.toString();
	}			
		
	/** 
	 * Generates a function which returns the current preference values			
	 */
	public String generateGetCurrentValuesFunction(){
		script = new StringBuffer("var current = new Array();");	
		// assemble the values
		script.append("function getCurrentValues() {");		
		script.append("for (var i = 0; i < 6; i++) {current[i] = currentValues[i];}");
		// find the first colour of the current palette
		script.append("for (var i = 0; i < palettes.length; i++) {");		
		script.append("if (palettes[i][palettes[i].length - 1] == current[0]) {");		
		script.append(" current[6] = palettes[i][0];}}");			
		script.append("return current;");
		script.append("} ");
		
		return script.toString();
	}
		
	/** 
	 * Generates an event handler which toggles the visibility attribute of the group/element 
	 * with the specified "id". 
	 * 
	 * @param type the graphic type
	 */
	public String generateToggleVisibilityFunction(int sets, String type){
		script = new StringBuffer("var values=1, percents=1;var textboxState = new Array();");
		script.append("for (var i = 0; i < ").append(sets).append("; i++) {textboxState[i] = 0;}");
		
		if (type == IGraphicTypeConstants.PIE_CHART) {
			// set all object visibility to the specified value
			script.append("function toggleVisibility(id) {");	
			script.append("  setVisibility('tickline', 'hidden');");
			script.append("  if (isVisible('bothValues')) {");
			script.append("    setVisibility('bothValues', 'hidden');");
			script.append("  } else if (isVisible('percentValues')) {");
			script.append("	   setVisibility('percentValues', 'hidden');");
			script.append("  } else if (isVisible('exactValues')) {");
			script.append("	   setVisibility('exactValues', 'hidden');");
			script.append("  } else {");
			script.append("    setVisibility('tickline', 'visible');");
			script.append("    if ((values == 1 && percents == 1) || (values == 0 && percents == 0)) { ");
			script.append("      setVisibility('bothValues', 'visible');");
			script.append("    } else if (values == 1) {");
			script.append("	     setVisibility('exactValues', 'visible');");
			script.append("    } else if (percents == 1) {");
			script.append("	     setVisibility('percentValues', 'visible');");
			script.append("    }");
			script.append("  }");	
			script.append("}");
		}
		else if (type == IGraphicTypeConstants.LINE_CHART) {		
			script.append("var noShapes = true;");			
			script.append("function changeAllTextBoxes(state) {");
			script.append("  for (var i = 0; i < ").append(sets).append("; i++) {");
			script.append("    var boxId = \"textbox\" + i;");
			script.append("    if (document.getElementById(boxId) != null) {");
			script.append("      if (isVisible('dataColour'+i)){setVisibility(boxId, state);}");
			script.append("      if (state == 'hidden') {textboxState[i] = 0;}");
			script.append("      else if (state == 'visible') {textboxState[i] = 1;}");
			script.append("    }");
			script.append("  }");
			script.append("}");
			script.append("function toggleVisibility(id) {");
			script.append("  if (id.substring(0,10) == 'dataColour') {");
			script.append("	   var lineId = id.substring(10,id.length);");
			script.append("    if (isVisible(\"dataColour\" + lineId) ) {");
			script.append(" 	 setVisibility(\"dataColour\" + lineId, 'hidden');");
			script.append("		 setVisibility(\"textbox\" + lineId, 'hidden');");
			script.append("	   } else if (!isVisible(\"dataColour\" + lineId)) {");
			script.append("	     setVisibility(\"dataColour\" + lineId, 'visible');");
			script.append("		 var state = textboxState[lineId] == 0 ? 'hidden' : 'visible';	");		
			script.append("		 setVisibility(\"textbox\" + lineId, state);");			
			script.append("	   }");
			// turn on/off the shapes on a line
			script.append("    var group = document.getElementById(id);");
			script.append("    if (group != null) {");
			script.append("      var useElementGroup = group.getElementsByTagName('use');");
			script.append("      if (useElementGroup != null) {");
			script.append("        for (var j = 0; j < useElementGroup.length; j++) {");
			script.append("          useElementGroup.item(j).setAttribute('visibility', (isVisible(id) && noShapes == false ? 'visible':'hidden') );");
			script.append("        }");
			script.append("      }");
			script.append("    }");
			script.append("  } else if (id.substring(0,7) == 'textbox') {");
			script.append("    if (isVisible(\"dataColour\" + id.substring(7,id.length))) {");
			script.append("      changeAllTextBoxes(isVisible(id) ? 'hidden':'visible');");
			script.append("    }");	
			script.append("  }");	
			script.append("}");		
		}
		else if (type == IGraphicTypeConstants.STACKBAR_CHART) {					
			script.append("function toggleVisibility(id) {");
			script.append("  setVisibility('exactValues', isVisible('exactValues')?'hidden':'visible');");
			script.append("  for (var i = 0; i < ").append(sets).append("; i++) {");  // turn on/off all boxes
			script.append("    var textBoxSet = document.getElementById(\"textbox\" + i);");
			script.append("    if (textBoxSet != null) { ");
			script.append("      textBoxSet.setAttribute('visibility', isVisible('exactValues')?'visible':'hidden');");
			script.append("	   }");
			script.append("	 }");
			script.append("}");		
		}
		else {				
			script.append("function toggleVisibility(id) {");
			script.append(" if(isVisible(id)){setVisibility(id,'hidden');}");
			script.append(" else{setVisibility(id,'visible');}");
			script.append("}");
		}		
		
		// check if an object is visible
		script.append("function isVisible(id) {");
		script.append(" if (document.getElementById(id).getAttribute(\"visibility\") == \"visible\") {return true;}");
		script.append("} ");
		
		// set a particular object visibility
		script.append("function setVisibility(id, value) {");
		script.append(" document.getElementById(id).setAttribute(\"visibility\",value);");
		script.append("} ");		
			
		return script.toString();
	}
		
	/**
	 * Generates helper functions used for XY-style charts.
	 */	
	public String generateXYChartHelperFunctions() {
		script = new StringBuffer("function resizeAndRepositionTextBox(textBox, newX, newY, newWidth, newHeight) {");				
		script.append(" textBox.setAttribute('x', newX);");
		script.append(" textBox.setAttribute('y', newY);");
		script.append("	textBox.setAttribute('width', newWidth);");
		script.append("	textBox.setAttribute('height', newHeight);");
		script.append("} ");
		
		script.append("function resizeTextBox(textBox, newWidth, newHeight) {");				
		script.append("	textBox.setAttribute('width', newWidth);");
		script.append("	textBox.setAttribute('height', newHeight);");
		script.append("} ");
		
		script.append("function repositionText(text, newX, newY) {");	
		script.append("	text.setAttribute('x', newX);");
		script.append("	text.setAttribute('y', newY);");
		script.append("} ");
				
		return script.toString();
	}
					
	/**
	 * Generates helper functions used for pie charts.
	 */				
	public String generatePieChartHelperFunctions() {
		script = new StringBuffer("");		
		script.append("function changeTextColour(bothElement, percentTextElement, exactTextElement, palette) {");
		script.append("  for (var i = 0, j = palette.length - 1; i < percentTextElement.length; i++, j = palette.length - 1) {");
		script.append("    if (((i+1) % j) == 0) {j = palette.length - 2;} else {j=((i+1)%j)-1;}"); 
		script.append("    bothElement.item(i).setAttribute('style', \"fill:\" + palette[j]);");
		script.append("	   percentTextElement.item(i).setAttribute('style', \"fill:\" + palette[j]);");
		script.append("	   exactTextElement.item(i).setAttribute('style', \"fill:\" + palette[j]);");
		script.append("  }");
		script.append("} ");
			
		return script.toString();						
	}	
	
	/** 
	 * Generates an event handler which resets the graphic user preference styles
	 * to their former values. 
	 */
	public String generateResetStylesFunction(){		
		script = new StringBuffer("function resetStyles(){");			
		script.append("updateStyles(currentValues, true);");					
		script.append("} ");
		
		return script.toString();
	}	
	
	/** 
	 * Generates an event handler which updates the graphic legend with user preference styles.
	 *  	
	 * @param sets 		the number of data sets displayed by the graphic legend
	 * @param type 		the graphic type
	 */
	public String generateUpdateLegendStylesFunction(int sets, String type){		
		script = new StringBuffer();
		script.append("function updateLegendStyles(newStyles, preview) {");

		script.append("var fontSize=parseInt(newStyles[1]);");
		// legend title
		script.append("var group = document.getElementById(\"legendTitle\");");
		script.append("if (group != null){");
		script.append(" group.setAttribute(\"style\",\"font-size:\" + (fontSize + 2) + \"pt; \" + newStyles[2]);}");
		// legend data
		script.append("var group = document.getElementById(\"legendData\");");
		script.append("if (group != null){");
		script.append(" group.setAttribute(\"style\",\"font-size:\" + fontSize + \"pt; \" + newStyles[2]);}");
		// only line chart has shapes		
		if (type == IGraphicTypeConstants.LINE_CHART)  {		
			script.append("for (var i = 0; i < " + sets + "; i++){");			
			script.append("  var squareshape = document.getElementById('legendsquareshapeWithoutLine' + i);");
			script.append("  var shape = document.getElementById('shape' + i);");		
			script.append("  shape.setAttribute('visibility', (newStyles[3] == '0' ? 'hidden' : 'visible'));");		
			script.append("  squareshape.setAttribute('visibility', (newStyles[3] == '0' ? 'visible' : 'hidden'));");
			script.append("}");
		}
		script.append("}");
		return script.toString();
		
	}	
	
	/** 
	 * Generates an event handler which updates the graphic with user preference styles.
	 *  	
	 * @param sets 		the number of data sets displayed by the graphic
	 * @param type 		the graphic type
	 */
	public String generateUpdateStylesFunction(int sets, String type, boolean hasLegend){		
		script = new StringBuffer();
		script.append("function getTextElement(myGroup) { return myGroup.getElementsByTagName('text'); }");
		// newStyles is an array and looks like: {palette name, font size, font text, shape, showValues, showPercents} 
		script.append("function updateStyles(newStyles, preview) {");			
		script.append("if (newStyles == null || newStyles.length < 6) {return;}");
		// get the chosen colour palette
		script.append("var paletteName=newStyles[0];");
		// use the first one if we get messed up here
		script.append("var palette= palettes[0];");		
		script.append("for (var i = 0; i < palettes.length; i++) {");
		script.append("if (palettes[i][palettes[i].length-1] == paletteName) {");
		script.append(" palette = palettes[i];}}");
		// update the color groups with the new palette, wrapping if not enough colours		
		script.append("for (var i=0, j=palette.length-1; i<" + sets + ";i++, j=palette.length-1) {");
		script.append("  if (((i+1) % j) == 0) {j = palette.length - 2;} else {j=((i+1)%j)-1;}"); 
		script.append("  var group = document.getElementById(\"dataColour\" + i);");		
		script.append("  if (group != null){");
		script.append("    group.setAttribute(\"style\",\"stroke:\" + palette[j] + \"; fill:\" + palette[j]);}");
		script.append("  group = document.getElementById(\"legendsquareshapeWithoutLine\" + i);");
		script.append("  if (group != null){");
		script.append("    group.setAttribute(\"style\",\"fill:\" + palette[j]);}");		
		
		if (type == IGraphicTypeConstants.LINE_CHART) {						
			script.append("  group = document.getElementById(\"shape\" + i);");
			script.append("  if (group != null){");
			script.append("    group.setAttribute(\"style\",\"fill:\" + palette[j]);}");		
		}
		
		script.append("  group = document.getElementById(\"pieColour\" + i);"); 
		script.append("  if (group != null){");  
		script.append("    group.setAttribute(\"style\", \"stroke:\" + \"black\" + \"; fill: \" + palette[j]);}"); 
		script.append("  group = document.getElementById(\"percentValues\" + i);"); 
		script.append("  if (group != null){");  
		script.append("    group.setAttribute(\"style\", \"stroke:\" + \"black\" + \"; fill: \" + palette[j]);}"); 
		script.append("  group = document.getElementById(\"bothValues\" + i);");
		script.append("  if (group != null){");
		script.append("    group.setAttribute(\"style\", \"stroke:\" + palette[j] + \"; fill: \" + palette[j]);}"); 
		script.append("  group = document.getElementById(\"line\" + i);");
		script.append("  if (group != null){");
		script.append("    group.setAttribute(\"style\", \"fill: \" + palette[j]);}");		
		script.append("}");
				
		// update the font group with the new font attributes, e.g., "size:11pt; weight:normal; 
		// family:Arial, sans-serif; stroke:#000000; "
		
		script.append("var fontSize=parseInt(newStyles[1]);");
		// chart title
		script.append("var group = document.getElementById(\"graphTitle\");");
		script.append("var graphTitleStyle = 'font-weight:bold; ' + newStyles[2].substring(newStyles[2].indexOf('font-style'), newStyles[2].length);");
		script.append("if (group != null){");
		script.append(" group.setAttribute(\"style\",\"font-size:\" + (fontSize+3) + \"pt; \" + graphTitleStyle);}");
		// timestamp
		script.append("var group = document.getElementById(\"timeStamp\");");
		script.append("if (group != null){");
		script.append(" group.setAttribute(\"style\",\"font-size:\" + fontSize + \"pt; \");");
		script.append(" group.setAttribute(\"y\", fontSize + 5);}");
				
		if (type != IGraphicTypeConstants.METER) {
			// legend specific
			if (hasLegend && type != IGraphicTypeConstants.BAR_CHART) {
				script.append("updateLegendStyles(newStyles, preview);");				
			}
			
			if (type != IGraphicTypeConstants.PIE_CHART) {
				// x-axis data
				script.append("var group = document.getElementById(\"dataValues0\");");
				script.append("if (group != null){");
				script.append(" group.setAttribute(\"style\",\"font-size:\" + fontSize + \"pt; \" + newStyles[2]);}");
				// y-axis data
				script.append("var group = document.getElementById(\"dataValues1\");");
				script.append("if (group != null){");
				script.append(" group.setAttribute(\"style\",\"font-size:\" + fontSize + \"pt; \" + newStyles[2]);}");
					
				// resize textboxes and text inside textboxes
				script.append("for (var i = 0; i < ").append(sets).append("; i++) {");
				script.append("var number = 0; var rectElement = null; var textElement = null;");					
				script.append("while ((rectElement = document.getElementById(\"textboxRect\" + i + number)) != null) {");				
				script.append("  textElement = document.getElementById(\"textboxText\" + i + number);");				
				script.append("	 var rectX = parseFloat(rectElement.getAttribute('x'));");
				script.append("	 var rectY = parseFloat(rectElement.getAttribute('y'));");		
				script.append("  var rectWidth = parseFloat(rectElement.getAttribute('width'));");
				script.append("  var rectHeight = parseFloat(rectElement.getAttribute('height'));");
				script.append("  var crossXAxisFlag = rectElement.getAttribute('crossXAxisFlag');");
				script.append("	 var defaultRectWidth = 16 + (parseInt(textElement.getAttribute('textLength')) - 1.0) * 6.0;");
				script.append("	 var defaultRectHeight = 18;");	
				script.append("  var textLength = parseInt(textElement.getAttribute('textLength'));");
				script.append("  textElement.setAttribute('style', 'font-size:' + fontSize + 'pt');");
		
				script.append("  var newTextBoxWidth, newTextBoxHeight, newRectY, newTextX, newTextY;");
				script.append("  if (fontSize == 7) {");
				script.append("    newTextBoxWidth = defaultRectWidth + (fontSize - 9) * 2;");
				script.append("    newTextBoxHeight = defaultRectHeight + (fontSize-9) * 2.8;");
				script.append("  } else if (fontSize == 8) {");
				script.append("    newTextBoxWidth = defaultRectWidth + (fontSize - 9) * 3;");
				script.append("    newTextBoxHeight = defaultRectHeight + (fontSize-9) * 3;");
				script.append("  } else if (fontSize == 9) {");
				script.append("    newTextBoxWidth = defaultRectWidth;");
				script.append("    newTextBoxHeight = defaultRectHeight - 1;");
				script.append("  } else if (fontSize == 10) {");
				script.append("    newTextBoxWidth = defaultRectWidth + (fontSize - 9) * 2;");
				script.append("    newTextBoxHeight = defaultRectHeight + 1;");
				script.append("  } else if (fontSize == 11) {");
				script.append("    newTextBoxWidth = defaultRectWidth + (fontSize - 9) * 3;");
				script.append("    newTextBoxHeight = defaultRectHeight + 1;");
				script.append("  }");
				script.append("  newRectY = crossXAxisFlag == '1' ? (rectY - (newTextBoxHeight - rectHeight)) : rectY;");
				script.append("  resizeAndRepositionTextBox(rectElement, rectX, (crossXAxisFlag == '1' ? newRectY : rectY), newTextBoxWidth, newTextBoxHeight);");
				script.append("  newTextX = rectX + newTextBoxWidth / 2.0;");
				script.append("  newTextY = newRectY + newTextBoxHeight / 2.0 + fontSize / 2.0;");
				script.append("  repositionText(textElement, newTextX, newTextY);");

				script.append("  number++;");				
				script.append("  }");  // end: for 
				script.append("}");  // end for 
			}			
			else {	
				// percentages
				script.append("var group = document.getElementById(\"percentValues\");");
				script.append("if (group != null){");
				script.append(" group.setAttribute(\"style\",\"font-size:\" + fontSize + \"pt; \");}");
				
				// both values 
				script.append("var bothGroup = document.getElementById(\"bothValues\");");
				script.append("if (bothGroup != null){");
				script.append(" bothGroup.setAttribute(\"style\",\"font-size:\" + fontSize + \"pt; \");}");	
			}
			
			// x-axis label
			script.append("var group = document.getElementById(\"labelsValues0\");");
			script.append("if (group != null){");
			script.append(" group.setAttribute(\"style\",\"text-anchor:middle; font-size:\" + (fontSize+2) + \"pt; \" + newStyles[2]);}");
			// y-axis label
			script.append("var group = document.getElementById(\"labelsValues1\");");
			script.append("if (group != null){");		
			script.append(" group.setAttribute(\"style\",\"font-size:\" + (fontSize+2) + \"pt; \" + newStyles[2]);}");		
			// exact values
			script.append("var group = document.getElementById(\"exactValues\");");
			script.append("if (group != null){");
			script.append(" group.setAttribute(\"style\",\"font-size:\" + fontSize + \"pt; \");}");
			
		}		
		else {
			// minmaxRate
			script.append("var group = document.getElementById(\"minmaxRate\");");
			script.append("if (group != null){");
			script.append(" group.setAttribute(\"style\",\"font-size:\" + (fontSize) + \"pt; \");}");
			// rateDefinition	
			script.append("var group = document.getElementById(\"rateDefinition\");");
			script.append("if (group != null){");
			script.append(" group.setAttribute(\"style\",\"font-size:\" + (fontSize) + \"pt; \");}");	
			// critDiv	
			script.append("var group = document.getElementById(\"critDiv\");");
			script.append("if (group != null){");
			script.append(" group.setAttribute(\"style\",\"font-size:\" + (fontSize) + \"pt; \");}");
			// meter label
			script.append("var group = document.getElementById(\"meterLabel\");");
			script.append("if (group != null){ ");
			script.append("   var tmpText = group.getElementsByTagName('text');");
			script.append("   for (var i = 0; i < tmpText.length; i++) {");
			script.append("     tmpText.item(i).setAttribute('style', 'font-size: ' + (fontSize) + 'pt; ');");
			script.append("   }");
			script.append("}");	
		}	
		
		// Show shapes
		if (type == IGraphicTypeConstants.LINE_CHART)  {
			script.append("for (var i = 0; i < " + sets + "; i++){");			
			script.append("  var group = document.getElementById('dataColour' + i);");
			script.append("  if (group != null) {");
			script.append("    var useElementGroup = group.getElementsByTagName('use');");
			script.append("    if (useElementGroup != null) {");
			script.append("      for (var j = 0; j < useElementGroup.length; j++) {");
			script.append("        useElementGroup.item(j).setAttribute('visibility', (newStyles[3] == '1' && isVisible('dataColour'+i) ? 'visible' : 'hidden') );");
			script.append("      }");
			script.append("    }");
			script.append("  }");
			script.append("  noShapes = newStyles[3] == '0' ? true : false;");
			script.append("}");
		}
		
		// show/hide values 
		if (type == IGraphicTypeConstants.PIE_CHART) {	
			script.append("var bothGroup=document.getElementById('bothValues');var perGroup=document.getElementById('percentValues');var eGroup=document.getElementById('exactValues');");
			script.append("changeTextColour(getTextElement(bothGroup), getTextElement(perGroup), getTextElement(eGroup), palette);");
			script.append("values = (newStyles[4] == '0' ? 0 : 1);");
			script.append("percents = (newStyles[5] == '0' ? 0 : 1);");
			script.append("setVisibility('tickline', (values==0 && percents==0 ? 'hidden':'visible'));");
			script.append("eGroup.setAttribute('visibility', (values==1 && percents==0 ? 'visible':'hidden'));");
			script.append("perGroup.setAttribute('visibility', (values==0 && percents==1 ? 'visible':'hidden'));");
			script.append("bothGroup.setAttribute('visibility', (values==1 && percents==1 ? 'visible':'hidden'));");			
		}
		else if (type == IGraphicTypeConstants.LINE_CHART) {										
			script.append("for (var i = 0; i < ").append(sets).append("; i++) {");
			script.append("  if (document.getElementById(\"textbox\" + i) != null) {");
			script.append("    textboxState[i] = parseInt(newStyles[4]);");
			script.append("    if (isVisible('dataColour'+i)) {setVisibility(\"textbox\"+i, newStyles[4]=='0' ? 'hidden':'visible');}");
			script.append("	 }");
			script.append("}");
		}
		else if (type == IGraphicTypeConstants.STACKBAR_CHART) {		
			script.append("var showAllState = newStyles[4]=='0' ? 'hidden':'visible';");
			script.append("var eGroup = document.getElementById('exactValues');");
			script.append("if (eGroup!=null) {eGroup.setAttribute('visibility',showAllState);}");
			script.append("for (var i = 0; i < ").append(sets).append("; i++) {");
			script.append("  if (document.getElementById(\"textbox\" + i) != null) {");
			script.append("    setVisibility(\"textbox\"+i, showAllState);");
			script.append("  }");
			script.append("}");
		}
		else if (type != IGraphicTypeConstants.METER) {			
			script.append("var eGroup = document.getElementById(\"exactValues\");");
			script.append("if (eGroup != null){");
 			script.append("eGroup.setAttribute('visibility', (newStyles[4]=='0'? 'hidden':'visible') );");
 	        script.append("}");			
		}
	
		// update current values if this is not just a preview
		script.append("if (!preview) {for(var i=0; i<6; i++) {currentValues[i] = newStyles[i];}");
		script.append("storePreferences(currentValues);}");		
		script.append("} ");
		
		return script.toString();	
	}	
	
	/** 
	 * Generates an event handler which retrieves user preference styles from cookie
	 *  	
	 * @param type 		the graphic type
	 */
	public String generateGetPreferencesFunction(String type) {
		String cookie = type;
		
		script = new StringBuffer("function getPreferences() {");		
		script.append("var value = get_Cookie(\"");
		script.append(cookie);
		script.append("\");");		
		script.append("if (value != null) {");
		script.append("return value.split(\"&\");");
		script.append("} else {");
		script.append("return null;}");
		script.append("}");
			
		return script.toString();
	}
		
	/** 
	 * Generates an event handler which stores user preference styles in cookie
	 *  	
	 * @param type 		the graphic type
	 */
	public String generateStorePreferencesFunction(String type) {
		String cookie = type;		
		
		script = new StringBuffer("function storePreferences(storevalues) {");
		script.append("var value = '';");
		script.append("for (var i = 0; i < storevalues.length -1; i++) {");
		script.append("  value += storevalues[i] + \"&\";");
		script.append("}");
		script.append("value += storevalues[storevalues.length-1];");
		script.append("set_Cookie('"); 
		script.append(cookie); 
		script.append("', value, expires_date, '/');");		
		script.append("}");
	
		return script.toString();
	}
	
	/** 
	 * Generates an event handler which allows the object attributes to
	 * be returned as a string for debugging purposes, for example, in
	 * an "alert".  
	 */
	public String generateInspectFunction(){
		script = new StringBuffer("function inspect(obj)");
		script.append("{var output=\"\";");
        script.append("if(obj==null){");
        script.append("return null;} ");
        script.append("else{");
        script.append("for(var i in obj){");
        script.append(" output += i + \" : \" + obj[i] + \"\\n\";}");
        script.append("return output;}");
        script.append("} ");
		
		return script.toString();
	}
	
	/** 
	 * Generates an event handler for reading and writing cookies.  
	 */
	public String generateCookieFunctions(){ 
		// note that escape() has been deprecated in ECMAScript v3 and
		// may not work in IE 5.5+ and Netscape 6/Mozilla
		script = new StringBuffer("var today= new Date();"); 	
		// set expiry date for one year from current date	
 		script.append("var expires_date= new Date(today.getTime() + (365 * 24 * 60 * 60 * 1000));");
		
		script.append("function get_Cookie(name){");	
		// can't access browser DOM in NN - have to use window function calls 
		// hasSVGCookieSupport and get_/set_NNCookie have to be defined in embedding HTML
		script.append("if (isNN && hasSVGCookieSupport) {");
		script.append(" return get_NNCookie(name);");	
		script.append("} else if (isIE && top.document.cookie != null) {");
		script.append("	var start = top.document.cookie.indexOf(name+\"=\");");
		script.append("	var len = start+name.length+1;");
		script.append("	if ((!start) && (name != top.document.cookie.substring(0,name.length))) return null;");
		script.append("	if (start == -1) return null;");
		script.append("	var end = top.document.cookie.indexOf(\";\",len);");
		script.append("	if (end == -1) end = top.document.cookie.length;");
		script.append("	return unescape(top.document.cookie.substring(len,end));");
		script.append("} else {");
		script.append(" return null;");
		script.append("}");
		script.append("} ");
			
		script.append("function set_Cookie(name,value,expires,path,domain,secure){");
		// can't access browser DOM in NN - have to use window function calls 
		script.append("if (isNN && hasSVGCookieSupport) {");
		script.append(" set_NNCookie(name,value,expires,path,domain,secure);");
		script.append("} else if (isIE) { ");
		script.append("top.document.cookie = name + \"=\" +escape(value) + ");
		script.append("((expires) ? \";expires=\" + expires.toGMTString() : \"\") +");
		script.append("((path) ? \";path=\" + path : \"\") + ");
		script.append("((domain) ? \";domain=\" + domain : \"\") + ");
		script.append("((secure) ? \";secure\" : \"\");" );
		script.append("}");	  
		script.append("}");	
    	
		return script.toString();
	}
		
}
