package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.util.Hashtable;

import org.w3c.dom.Document;

/**
 * Data retrieval methods which retrieve data of interest to <code>SVGMeter</code> from 
 * documents which conform to the documented data input schemas. 
 * 
 * @version %I%
 */
public class MeterDataRetriever extends CommonDataRetriever {
	/**
	 * Sole constructor.
	 */			
	MeterDataRetriever() {
		super();
	}		
		
	/**
	 * Returns from the data document the text labels which indicate the 
	 * contextual definition of the rate and its actual value, both of
	 * which are displayed by the meter graphic.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_CONTEXT_FOUND, 
	 * NO_SUMMARY_FOUND</code> 
	 * @see IDataInputProcessingExceptionCodes
	 */	
	public Hashtable getLabels() throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		Document document = (configDocument != null) ? configDocument : dataDocument;
		Hashtable labels = new Hashtable(2);		
		
		try {			
			labels.put(RATE_DEFINITION, getAttributeFromSingleElement(dataDocument, RANGE, CONTEXT));							
		} catch (Exception exc) {
			// NullPointerException
			throwException(NO_CONTEXT_FOUND, RANGE);			
		}	
		try {			
			labels.put(ACTUAL_RATE, getAttributeFromSingleElement(dataDocument, DATAUPDATE, SUMMARY));							
		} catch (Exception exc) {
			// NullPointerException
			throwException(NO_SUMMARY_FOUND, DATAUPDATE);			
		}				
		return labels;
	}	
		
	/**
	 * Returns from the data document the value to which the meter needle points.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_VALUE_FOUND, INVALID_NUMBER_FORMAT</code> 	 
	 * @see IDataInputProcessingExceptionCodes
	 */	
	public String getNeedleValue() throws DataInputProcessingException {	
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
			
		// get the first datapoint value from the first dataset,ignoring position
		String result = getAttributeValueFromFirstChildOfFirstElement(dataDocument, DATASET, DATAPOINT, VALUE);	
		if (result == null || result.equals(EMPTY_STRING)) {
			throwException(NO_VALUE_FOUND, DATAPOINT);	
		}	
		// ensure it's a number
		double j = 0;
		try {
			j = Double.parseDouble(result);
		} catch (Exception exc) {
			// NumberFormatException
			throwException(INVALID_NUMBER_FORMAT, DATAPOINT);	
		}
		// restriction to non-negative integers
			if (j < 0) {
				throwException(INVALID_NUMBER_FORMAT, DATAPOINT);
			}			
		return result;				
	}		
}

