package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.Hashtable;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
/**
 * <code>SVGBarChart</code> generates a bar chart graphic DOM using Scalable Vector Graphics (SVG).
 * 
 * @version %I%
 */
public class SVGBarChart extends SVGXYChart implements IGraphicDocumentStyle, IDataInputProcessingExceptionCodes, Serializable {
	/**
	 * Sole constructor.
	 */
	public SVGBarChart() {
		super();
		graphicType = IGraphicTypeConstants.BAR_CHART;
		setPreferencesPage(BAR_PREFS);
		setPreferencesPageWidth(PREFS_WIDTH);
		setPreferencesPageHeight(BAR_PREFS_HEIGHT);		
	}
	
	/**
	 * @see IGraphicDocumentGenerator#generateGraphicDocument()
	 */	
	public Document generateGraphicDocument() throws DataInputProcessingException {							
		String [] rangeMarkers = null;
		String [] categories = null;	
		String [] barValues = null;
		String rangeMaxValue = null;
		int xTicks, yTicks;
		
		// make sure there is some data before continuing		
  		try {
			barValues = dataRetriever.getDatapointValuesFromFirstDataset();					
  		} catch (DataInputProcessingException e) {
  			e.setWidgetType(graphicType);
	  		//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());	
	  		throw e; 			
  		}			
        
		// !DO NOT REORDER THESE METHOD CALLS UNLESS NECESSARY!!
		createSvgDocument(dataDocument.getImplementation());
		
		// add the "onload" event handler function
		svgRoot.setAttribute("onload", "init(\"bar\")");
				
		addJavaScriptFunctions(svgRoot, barValues.length);
 		addDefinitions(svgRoot, barValues.length);
	    addBorder(svgRoot);
  
  		// get all the rest of the data and add it 
  		// if something important is missing, exit
		try {
			rangeMarkers = dataRetriever.getRangeMarkers();
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;				
		}					
		
		if (rangeMarkers != null && rangeMarkers.length > 0) { 
			rangeMaxValue = (String)rangeMarkers[rangeMarkers.length - 1];
		}			
		yTicks = rangeMarkers.length;
		
		try {
			categories = dataRetriever.getCategories();
			if (categories.length != barValues.length) {
				// missing category 
				short code = MISSING_CATEGORY_NAMES;				
				String element = CATEGORY;				
				DataInputProcessingException e = new DataInputProcessingException(code, element);
				throw e;
			}
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;				
		}			
		xTicks = categories.length;
       
        addGrid(svgRoot, xTicks, yTicks, rangeMarkers);
        addAxisLabels(svgRoot, categories, rangeMarkers);
        addDataSet(svgRoot, barValues, xTicks, rangeMaxValue);	
		
		try {
			Hashtable titles = dataRetriever.getTitles();		
 		    addTitles(svgRoot, titles);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());			
			throw e;
		}
		
		try {
			String timestamp = dataRetriever.getTimestamp();	
        	addTimeStamp(svgRoot, timestamp);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;
		}
		                         
		return super.generateGraphicDocument();
	}
		
	// add the data value bars	
	private void addBars(Element parent, Element toggleParent, double yUnits, String [] input, int xTicks) {
		int numberOfBars = input.length; 
		// at coordinates (0,0)
		double xZero = GRIDXOFFSET;  			
		double yZero = GRIDYOFFSET + yAxisLength - 1;  
		double xIntervalLength = (double)xAxisLength/xTicks;  // each x interval length
		
		for (int i = 0; i< numberOfBars; i++) {
			String barValue = input[i];					// bar value
			Element newElement = generatedDocument.createElement("rect");
			double barWidth = xIntervalLength * 0.5;    // width of the bar
			// NumberFormatException not possible here
			double barHeight = yUnits * Double.parseDouble(barValue) - 0.9;		 // height of the bar
			double barX = xZero + xIntervalLength * (((double)i) + 0.5) - barWidth * 0.5;  // bar x position
			double barY = yZero - barHeight;  // bar y position
	
			newElement.setAttribute("id", "dataColour" + i);  // for updating colour change
			newElement.setAttribute("class", "shape" + i);
			newElement.setAttribute("x", Double.toString(barX));
			newElement.setAttribute("y", Double.toString(barY));
			newElement.setAttribute("width", Double.toString(barWidth));
			newElement.setAttribute("height", Double.toString(barHeight));
			newElement.setAttribute("stroke-width", "0.75pt");
			newElement.setAttribute("stroke", "#666666");			
			parent.appendChild(newElement);
			
			// Set up the position of the text over a bar			
			double textX = barX + barWidth / 2.0;
			double textY = barY - 5.0;   // text y position
			
			// Add values over the bar			
			addExactValue(toggleParent, textX, textY, barValue);
		}			
	}
			
	// add the data value-related elements
	private void addDataSet(Element parent, String [] yValues, int xTicks, String yMax) {		
		Element gToggleElement = generatedDocument.createElement("g");
		String stylesheetClass = "dataValues anchorAtMiddle";		
		gToggleElement.setAttribute("id","exactValues");
		gToggleElement.setAttribute("class", stylesheetClass);		
		gToggleElement.setAttribute("font-family", getGraphicFont());
		gToggleElement.setAttribute("visibility", "hidden");
		gToggleElement.setAttribute("onclick","toggleVisibility(\"exactValues\")");
		parent.appendChild(gToggleElement);
		
		Element gDataSetElement = generatedDocument.createElement("g");
		gDataSetElement.setAttribute("id","dataset");
		gDataSetElement.setAttribute("onclick","toggleVisibility(\"exactValues\")");
		parent.appendChild(gDataSetElement);
		
		// need to compute y values for actual datapoints based on the number of pixels 
		// between each potential datapoint along y-axis
		// NumberFormatException not possible here
		double yLabelsMaxValue = Double.parseDouble(yMax);			
		addBars(gDataSetElement, gToggleElement, (yAxisLength/yLabelsMaxValue), yValues, xTicks);		
	}
	
	// add default style sheet
	private void addDefaultInternalStylesheet(Element parent, int sets) {
		StringBuffer styleRules = new StringBuffer(COMMON_STYLE);
		styleRules.append(" ");
		styleRules.append(XYCHART_STYLE);		
				
		String [] palette = (String [])graphicColourPalette.get(graphicColourPaletteName);
		// palette has limited number of colours, so loop if more sets
		int paletteLength = palette.length;		
		for (int i = 0, j = 0; i < sets; i++) {
		  j = (i + 1) % paletteLength;
		  j = (j == 0) ? paletteLength - 1 : j-1;			
		  styleRules.append(" .dataset" + i + "{stroke-width:0.75pt; stroke:" + palette[j] + ";}"); 
		  styleRules.append(" .shape" + i + "{stroke:none; fill:" + palette[j] + ";}");
		}		
		CDATASection newCDATASection = generatedDocument.createCDATASection(styleRules.toString());
		
		Element newElement = generatedDocument.createElement("style");
		newElement.setAttribute("id", "defaultStyleRules");
		newElement.setAttribute("type", "text/css");
		newElement.appendChild(newCDATASection);
		parent.appendChild(newElement);
	}	
	
	// add all definitions
	private void addDefinitions(Element parent, int numberOfDatasets) {
		Element defsElement = generatedDocument.createElement("defs");
        parent.appendChild(defsElement);
        
        // if no stylesheet specified, use our default internal style rules
        //if (userStylesheet == null) {
		  addDefaultInternalStylesheet(defsElement, numberOfDatasets);		
		//}
				
		addPreferenceIcon(defsElement);
	}	
	
	// add category total values over the bar
	private void addExactValue(Element parent, double x, double y, String value) {				
		Element gElement = generatedDocument.createElement("g");
		gElement.setAttribute("id","text1");		
		addLabel(gElement, value, "dataValues, anchorAtMiddle", x, y, 0);
		parent.appendChild(gElement);	
	}	
}
