package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

/**
 * Convenience class for creating SVG document generators.
 * 
 * @version %I%
 * @see IGraphicDocumentGenerator 
 */
public abstract class SVGDocumentGeneratorFactory {
	/**
	 * Returns the <code>IGraphicDocumentGenerator</code> interface to an object 
	 * which generates the specified graphic type or <code>null</code> if the 
	 * type is not recognized.
	 * 
	 * @param generatorType one of the recognized graphic types
     * @return IGraphicDocumentGenerator the instantiated object	
     * @see IGraphicDocumentGenerator
     * @see IGraphicTypeConstants
     */
	public static IGraphicDocumentGenerator getGraphicDocumentGenerator(String generatorType) {
		IGraphicDocumentGenerator svgGenerator = null;
		
		if (generatorType.equals(IGraphicTypeConstants.LINE_CHART))
			svgGenerator = (IGraphicDocumentGenerator) new SVGLineChart();
		else if (generatorType.equals(IGraphicTypeConstants.PIE_CHART))
			svgGenerator = (IGraphicDocumentGenerator) new SVGPieChart();
		else if (generatorType.equals(IGraphicTypeConstants.BAR_CHART))
			svgGenerator = (IGraphicDocumentGenerator) new SVGBarChart();	
		else if (generatorType.equals(IGraphicTypeConstants.STACKBAR_CHART))
			svgGenerator = (IGraphicDocumentGenerator) new SVGStackBarChart();	
		else if (generatorType.equals(IGraphicTypeConstants.METER))
			svgGenerator = (IGraphicDocumentGenerator) new SVGMeter();	
			
		return svgGenerator;	
	}

}

