package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.util.Hashtable;
import java.util.Vector;

import org.w3c.dom.Document;
/**
 * Data retrieval methods which retrieve data of interest to <code>SVGLineChart, 
 * SVGBarChart</code> and <code>SVGStackBarChart</code> from documents which 
 * conform to the documented data input schemas. 
 * 
 * @version %I%
 */
public class XYChartDataRetriever extends CommonDataRetriever {
	/**
	 * Sole constructor.
	 */				
	XYChartDataRetriever() {
		super();
	}
	
	/**
	 * Returns the text values which represent the x- and y-axis context labels.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_DESCRIPTION_FOUND, NO_CONTEXT_FOUND</code>.
	 * @see IDataInputProcessingExceptionCodes
	 */
	public Hashtable getTitles() throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		Document document = (configDocument != null) ? configDocument : dataDocument;
		Hashtable titles = new Hashtable(2);
		
		try {			
			titles.put(X_TITLE, getAttributeFromSingleElement(dataDocument, CATEGORIZATION, DESCRIPTION));			
		} catch (Exception exc) {
			// NullPointerException
			throwException(NO_DESCRIPTION_FOUND, CATEGORIZATION);			
		}	
		try {			
			titles.put(Y_TITLE, getAttributeFromSingleElement(dataDocument, RANGE, CONTEXT));			
		} catch (Exception exc) {
			// NullPointerException
			throwException(NO_CONTEXT_FOUND, RANGE);			
		}			
		return titles;
	}		
}	