package reports.svg;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.File;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;

public class XMLUtil
{
	public static String getAttribute(Element element, String tagName)
	{
		if((element == null) || (tagName == null))
			return null;
			
		String value = element.getAttribute(tagName);
		if((value == null) || (value.equals("")))
		{
			NodeList list = element.getElementsByTagName(tagName);
			for(int i=0, max=list.getLength(); i<max; i++)
			{
				Node textNode = list.item(i);
				Node firstChild = list.item(0).getFirstChild();
				if(firstChild != null)
				{
					String auxValue = firstChild.getNodeValue();
					if((auxValue != null) && textNode.getParentNode().equals(element))
					{
						value = auxValue;
						break;
					}
				}
			}
		}
		return value;
	}

	public static int getIntAttribute(Element element, String tagName)
	{
		try
		{
			return Integer.parseInt(getAttribute(element, tagName));
		}
		catch(NumberFormatException nfe)
		{
		}
		
		return 0;
	}
	
	public static NodeList getChildrenByTagName(Element element, String tagName)
	{
		if((element == null) || (tagName == null))
			return null;
			
		Vector v = new Vector();
		
		NodeList children = element.getChildNodes();
		for(int i=0, max=children.getLength(); i<max; i++)
		{
			if((children.item(i) instanceof Element) && tagName.equals(((Element)children.item(i)).getTagName()))
				v.add(children.item(i));
		}
		
		return createNodeList(v);
	}
	
	public static NodeList createNodeList(final Vector nodes)
	{
		if(nodes == null)
			return null;
			
		NodeList nodeList = new NodeList()
		{
			/**
			 * @see NodeList#getLength()
			 */
			public int getLength()
			{
				return nodes.size();
			}

			/**
			 * @see NodeList#item(int)
			 */
			public Node item(int index)
			{
				return (Node)nodes.get(index);
			}
		};
		
		return nodeList;
	}
	
	public static Document getXmlDom(String xmlFilePath)
	{
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			docBuilderFactory.setIgnoringComments(true);
			docBuilderFactory.setValidating(true);
	
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
	
			final File xmlFile = new File(xmlFilePath.trim());
	
	
			docBuilder.setErrorHandler(createErrorHandler(false));
			return (docBuilder.parse(xmlFile));
		}
		catch (Exception e)
		{
		}
	
		return null;
	}
	
	private static ErrorHandler createErrorHandler(final boolean logMessages)
	{
		ErrorHandler errorHandler = new ErrorHandler()
		{
			public void error(SAXParseException exception)
			{
				//exception.printStackTrace();
			}

			public void fatalError(SAXParseException exception)
			{
				exception.printStackTrace();
			}

			public void warning(SAXParseException exception)
			{
				//exception.printStackTrace();
			}
		};
		
		return errorHandler;
	}
}